/*
 *  basicOCR.c
 *  
 *
 *  Created by damiles on 18/11/08.
 *  Copyright 2008 Damiles. GPL License
 *
 */
#ifdef _CH_
#pragma package <opencv>
#endif

#ifndef _EiC
#include "cv.h"
#include "highgui.h"
#include "ml.h"
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#endif

#include "preprocessing.h"
#include "basicOCR.h"



void basicOCR::getData()
{
	IplImage* src_image;
	IplImage* p_prs_image;
	IplImage prs_image;
	CvMat row,data;
	char file[255];
	int i,j;
	for(i =0; i<classes; i++){
		for( j = 1; j< train_samples; j++){
			
			//Load file
			if(i<10)
				sprintf(file,"%s%d/%d%04d.png",ocr_training_path, i, i, j);
			else
				sprintf(file,"%s%c/%c%04d.png",ocr_training_path, i+55, i+55, j);
				// sprintf(file,"%s%c/%c%04d.png",ocr_training_path, i+87, i+87, j);
			src_image = cvLoadImage(file,0);
			if(!src_image){
				printf("Error: Cant load image %s\n", file);
				//exit(-1);
			}
			//process file
			// prs_image = preprocessing(src_image, size, size);
			p_prs_image = preprocessing(src_image, size, size);
			prs_image = *p_prs_image;
			
			//Set class label
			cvGetRow(trainClasses, &row, i*train_samples + j);
			cvSet(&row, cvRealScalar(i));
			//Set data 
			cvGetRow(trainData, &row, i*train_samples + j);

			IplImage* img = cvCreateImage( cvSize( size, size ), IPL_DEPTH_32F, 1 );
			//convert 8 bits image to 32 float image
			cvConvertScale(&prs_image, img, 0.0039215, 0);

			cvGetSubRect(img, &data, cvRect(0,0, size,size));
			
			CvMat row_header, *row1;
			//convert data matrix sizexsize to vecor
			row1 = cvReshape( &data, &row_header, 0, 1 );
			cvCopy(row1, &row, NULL);
		}
	}
}

void basicOCR::train()
{
	knn=new CvKNearest( trainData, trainClasses, 0, false, K );
}

float basicOCR::classify(IplImage* img, int showResult)
{
	IplImage* p_prs_image;
	IplImage prs_image;
	CvMat data;
	CvMat* nearest=cvCreateMat(1,K,CV_32FC1);
	float result;

	//process file
	// prs_image = preprocessing(img, size, size);
	p_prs_image = preprocessing(img, size, size);
	prs_image = *p_prs_image;
	
	//Set data 
	IplImage* img32 = cvCreateImage( cvSize( size, size ), IPL_DEPTH_32F, 1 );
	cvConvertScale(&prs_image, img32, 0.0039215, 0);
	cvGetSubRect(img32, &data, cvRect(0,0, size,size));
	CvMat row_header, *row1;
	row1 = cvReshape( &data, &row_header, 0, 1 );

	result=knn->find_nearest(row1,K,0,0,nearest,0);
	
	int accuracy=0;
	for(int i=0;i<K;i++){
		if( nearest->data.fl[i] == result)
                    accuracy++;
	}
	float pre=100*((float)accuracy/(float)K);

	// Manual corrections
	if(result == 6 && accuracy <= 4)
		result = 0;

	// Debug output on request
	if(showResult==1) {
		printf("|\t%.0f\t| \t%.2f%%  \t| \t%d of %d \t| \n",result,pre,accuracy,K);
		printf(" ---------------------------------------------------------------\n");
	} else {
	  if(showResult==2) {
		if(result < 10) {
			printf("%c\n", (int)result+48);
                } else {
			printf("%c\n", (int)result+55);
                }

	  }
	}
	cvReleaseImage(&img32);
	cvReleaseImage(&p_prs_image);

	return result;
}


void basicOCR::test(){
	IplImage* src_image;
	IplImage* p_prs_image;
	IplImage prs_image;
	CvMat row,data;
	char file[255];
	int i,j;
	int error=0;
	int testCount=0;
	for(i =0; i<classes; i++){
		for( j = 31; j< 30+train_samples; j++){
			
			if(i<10)
				sprintf(file,"%s%d/%d%04d.png",ocr_training_path, i, i, j);
			else
				sprintf(file,"%s%c/%c%04d.png",ocr_training_path, i+55, i+55, j);
				// sprintf(file,"%s%c/%c%04d.png",ocr_training_path, i+87, i+87, j);
			src_image = cvLoadImage(file,0);
			if(!src_image){
				printf("Error: Cant load image %s\n", file);
				//exit(-1);
			}
			//process file
			// prs_image = preprocessing(src_image, size, size);
			p_prs_image = preprocessing(src_image, size, size);
			prs_image = *p_prs_image;
			float r=classify(&prs_image,0);
			if((int)r!=i)
				error++;
			
			testCount++;
		}
	}
	float totalerror=100*(float)error/(float)testCount;
/*
	printf("System Error: %.2f%%\n", totalerror);
*/
	
}

basicOCR::basicOCR(char * file_path)
{

	// Initial
	// sprintf(ocr_training_path , "../OCR/");
	sprintf(ocr_training_path , file_path);
	train_samples = 30;
	// classes= 10;
	classes= 16;
	size=40;

	printf("basicOCR: Begin training (from path: \"%s\", train samples: %d)\n", ocr_training_path, train_samples);
	trainData = cvCreateMat(train_samples*classes, size*size, CV_32FC1);
	trainClasses = cvCreateMat(train_samples*classes, 1, CV_32FC1);

	// Get data (get images and process it)
	getData();
	
	// train	
	train();
	// Test	
	test();
/*	
	printf(" ---------------------------------------------------------------\n");
	printf("|\tClass\t|\tPrecision\t|\tAccuracy\t|\n");
	printf(" ---------------------------------------------------------------\n");
*/
	printf("basicOCR: Training complete.\n");
}


