#ifdef _CH_
#pragma package <opencv>
#endif

#ifndef _EiC
#include <cv.h>
#include <highgui.h>
#include <cvaux.h>
#include <stdio.h>
#include <time.h>
#include <ctype.h>
#include "basicOCR.h"
#include "node.h"
#endif
#include <fstream>

using std::cout;
using std::endl;

// using namespace cv;
// using namespace std;


const char *VENTANA="clientlessTGXf Receiver";

CvCapture* capture = 0;
IplImage* imagen;
IplImage* screenBuffer;
IplImage* imagen_color;
IplImage* smooth;
IplImage* threshold;
IplImage* open_morf;
IplImage* blobResult;
IplImage* img_contornos;
IplImage* imgocr;
CvMat imagen_data;
CvMat imgocr_data;
CvMoments moments;
CvHuMoments humoments;
CvSeq* contour;
CvSeq* contourLow;
int red,green,blue;
int r,last_x, last_y;
int min_char_pixels_wide = 5;
int max_char_pixels_wide = 20;
int min_char_pixels_high = 10;
int max_char_pixels_high = 20;

int main( int argc, char** argv ) {
  CvScalar avg;
  CvScalar avgStd;
  CvScalar color;
  CvRect rect;
  CvPoint pt1, pt2;

  // OCR SETUP
  float ocr_result;
  LinkedList character_list;
  Node *chr = NULL;
  int alt_list = 0;
  LinkedList tgxf_list;

  // TGXf Setup
  int last_data_counter = -1;
  int expected_counter = 0;
  int lost_frames = 0;
  int data_counter = 0;
  ofstream output_file;
  char * output_buffer;

  srand ( time(NULL) );

  // Welcome
  printf("clientlessTGXf - Receiver - Proof-of-Concept by Ian Latter, MidnightCode.org\n"
    "\n"
    "  With much thanks to David Millan Escriva | Damiles\n"
    "\n"
    "    Basic OCR\n"
    "      Tutorial: http://blog.damiles.com/2008/11/basic-ocr-in-opencv/\n"
    "      Source:   https://github.com/damiles/basicOCR/\n"
    "\n"
    "    Blob and Contour Detection;\n"
    "      Tutorial: http://blog.damiles.com/2010/12/segmentation-and-feature-extraction-contours-and-blob-detection/\n"
    "      Source:   https://github.com/damiles/Segmentation-Tutorial\n"
    "\n");


  // Check if user specify image to process
  if(argc != 4 ) {
    printf("Use:\n\t%s ../OCR/ video.mp4 data.bin\n",argv[0]);
    return 0;
  }

  char* video_file = argv[2];
  char* data_file = argv[3];
  char* ocr_path = argv[1];

  // basicOCR ocr("../OCR/");
  basicOCR ocr(ocr_path);
  printf("\n");

  // Handle data output
  output_file.open(data_file, ios::out | ios::app | ios::binary); 
  if(!output_file.is_open()) {
    printf("Failed to open data file: %s\n", data_file);
    return -1;
  }
  output_buffer = new char[1];

  // Handle video input
  capture = cvCreateFileCapture(video_file);
  if(!capture) {
    printf("Failed to open video file: %s\n", video_file);
    return -1;
  }
  imagen_color = cvQueryFrame(capture);
  imagen = cvCreateImage(cvGetSize(imagen_color),IPL_DEPTH_8U,1);
  cvCvtColor(imagen_color,imagen,CV_BGR2GRAY);


  // Create needed images
  smooth= cvCreateImage(cvSize(imagen->width, imagen->height), IPL_DEPTH_8U, 1);
  threshold= cvCreateImage(cvSize(imagen->width, imagen->height), IPL_DEPTH_8U, 1);
  open_morf= cvCreateImage(cvSize(imagen->width, imagen->height), IPL_DEPTH_8U, 1);
	
  // Init variables for countours
  contour = 0;
  contourLow = 0;

  // Create window
  printf("Creating display window ..\n");
  cvNamedWindow(VENTANA, CV_WINDOW_NORMAL);

  while(1) {
    imagen_color = cvQueryFrame(capture);
    if(!imagen_color)
      break;

    cvCvtColor(imagen_color,imagen,CV_BGR2GRAY);

    // Smooth image
    cvSmooth(imagen, smooth, CV_GAUSSIAN, 3, 0, 0, 0);
    cvAvgSdv(smooth, &avg, &avgStd, NULL);
    // printf("Avg: %f\nStd: %f\n", avg.val[0], avgStd.val[0]);

    // Threshold image
    cvThreshold(smooth, threshold, (int)avg.val[0]-7*(int)(avgStd.val[0]/8), 255, CV_THRESH_BINARY_INV);

    // Morfologic filters
    cvErode(threshold, open_morf, NULL,1); 
    cvDilate(open_morf, open_morf, NULL,1); 

    // Duplicate image for countour
    img_contornos=cvCloneImage(open_morf);

    // Create storage needed for contour detection
    CvMemStorage* storage = cvCreateMemStorage(0);

    // Search countours in preprocesed image
    cvFindContours( img_contornos, storage, &contour, sizeof(CvContour),
      CV_RETR_EXTERNAL, CV_CHAIN_APPROX_SIMPLE, cvPoint(0, 0) );

    // Optimize contours, reduce points
    contourLow=cvApproxPoly(contour, sizeof(CvContour), storage,CV_POLY_APPROX_DP,1,1);
    cvReleaseImage(&img_contornos);

    // printf("-------------------------- [frame] --------------------------\n");
    for( ; contourLow != 0; contourLow = contourLow->h_next ) {
      color = CV_RGB( rand()&200, rand()&200, rand()&200 );
      rect = cvBoundingRect(contourLow, 0);
      if(rect.x > 0 && rect.y > 0 &&
        rect.width > min_char_pixels_wide && rect.height > min_char_pixels_high &&
        rect.width < max_char_pixels_wide && rect.height < max_char_pixels_high) {
        pt1.x = rect.x;
        pt2.x = (rect.x+rect.width);
        pt1.y = rect.y;
        pt2.y = (rect.y+rect.height);
        cvRectangle(imagen_color, pt1, pt2, color, 1, 8, 0); 

        // OCR
        imgocr = cvCreateImage(cvSize(rect.width,rect.height), IPL_DEPTH_8U, 1);
        cvGetSubRect(imagen, &imagen_data, rect);
        cvSet(imgocr, CV_RGB(255,255,255), NULL);
        cvGetSubRect(imgocr, &imgocr_data, cvRect(0,0,rect.width,rect.height));
        cvCopy(&imagen_data, &imgocr_data, NULL);
        ocr_result = ocr.classify(imgocr,0);
        if(ocr_result < 10) {
          // printf("Line %d, Col %d, %c\n", rect.y, rect.x, (int)ocr_result+48);
          character_list.add_char(rect.x, rect.y, (int)ocr_result+48);
        } else {
          // printf("Line %d, Col %d, %c\n", rect.y, rect.x, (int)ocr_result+55);
          character_list.add_char(rect.x, rect.y, (int)ocr_result+55);
        }
        cvReleaseImage(&imgocr);
      }
    }
    cvReleaseMemStorage(&storage);

    // Find TGXf Layer-3 Data
    character_list.mark_tgxf_rows();

    // Create TGXf Layer-4 Data
    int tot = 0;
    tot = character_list.populate_tgxf_list(tgxf_list);
    // printf("Total: %d\n", tot);
    character_list.empty_list();

    // Display TGXf Layer-4 Data
    /*
    while((chr = tgxf_list.pop_tgxf_octet()) != NULL) {
      printf("%c", chr->octet);
    }
    printf("\n");
    */


    // Process TGXf Layer-4 Data
    while((chr = tgxf_list.pop_tgxf_octet()) != NULL) {
      data_counter = chr->counter;
      expected_counter = last_data_counter + 1;
      if(expected_counter == 256)
        expected_counter = 0;
      // printf("{e %d,d %d;c %d}", expected_counter, data_counter, chr->counter);
      lost_frames = 0;
      if(data_counter != expected_counter) {
        // [..e........d..]
        if(data_counter > 230 && expected_counter < 20 && expected_counter < data_counter)
          continue;
        // [......d.e.....]
        if(expected_counter > data_counter && expected_counter - data_counter < 128)
          continue;
        // [......e.d.....]
        lost_frames = data_counter - expected_counter;
        // [..d........e..]
        if(expected_counter > data_counter)
          lost_frames = data_counter + 256 - expected_counter;
        // This should be structured to provide some assurance, but ..
        // there's a good chance that we'll get another pass at correcting this error on the next screen, so ..
        // ignore this octet onward in this screen.
	if(lost_frames > 0) {
          tgxf_list.empty_list();
          break;
        }
      }
      if(lost_frames > 0) {
        printf("[%d]", lost_frames);
      }
      // printf("%c", chr->octet);
      *output_buffer = chr->octet;
      // output_file.seekg(0, ios::end);
      output_file.write(output_buffer,1);
      last_data_counter = data_counter;
    }
    // ensure list is empty
    tgxf_list.empty_list();
/*
     // Write Data to File
      if(access((const char *)tgxf_session_file_name, F_OK) == -1) {
        if((fp = fopen((const char *)tgxf_session_file_name, "w+b")) == NULL) {
          fprintf(stdout, "ERROR(TGXf): unable to create/open file for reading/writing, at %s.\n", tgxf_session_file_name);
          break;
        }
      } else {
        if((fp = fopen((const char *)tgxf_session_file_name, "r+b")) == NULL) {
          fprintf(stdout, "ERROR(TGXf): unable to open file for reading/writing, at %s.\n", tgxf_session_file_name);
          break;
        }
        if(fseek(fp, (tgxf_session_qrcode_bytes * tgxf_session_frame_count), SEEK_SET) < 0) {
          fprintf(stdout, "ERROR(TGXf): unable to seek file, at %s.\n", tgxf_session_file_name);
          fclose(fp);
          break;
        }
      }
      if(fwrite((const void *)frame_payload, (size_t)bytes_to_use, (size_t)1, fp) != (size_t)1) {
        fprintf(stdout, "ERROR(TGXf): unable to write to file, at %s.\n", tgxf_session_file_name);
        fclose(fp);
        break;
      }
      fclose(fp);

      // Increment tally
      tgxf_session_last_data_counter = data_counter;
      tgxf_session_frame_count++;
*/


    

/*
 	int screen_row_y = -1;
        while((screen_row_y = character_list.get_nearest_y(screen_row_y)) > -1) {
          printf("Row: y=%d\n", screen_row_y);
 	  int screen_col_x = -1;
	  int col_count = 0;
          while((screen_col_x = character_list.get_nearest_x_for_y(screen_col_x, screen_row_y)) > -1) {
	    col_count++;
            printf("Ordered: x=%d y=%d\n", screen_col_x, screen_row_y);
          }
	  if(col_count == 16)
	    character_list.mark_row(screen_row_y);
	  col_count = 0;
        }
*/

    // double fps = cvGetCaptureProperty (capture,CV_CAP_PROP_FPS);
    // CvSize size = cvSize((int)cvGetCaptureProperty( capture, CV_CAP_PROP_FRAME_WIDTH),(int)cvGetCaptureProperty( capture, CV_CAP_PROP_FRAME_HEIGHT));

    // cvShowImage(VENTANA, imagen);
    cvShowImage(VENTANA, imagen_color);

    char c=cvWaitKey(30);
    if (c==27) break;
  }

  output_file.close();
  cvReleaseCapture( &capture );
  cvDestroyWindow(VENTANA);

  return 0;
}


